require 'securerandom'
require_relative './link'
require_relative './traversals'

module PatchworkInternal
  class Node
    attr_reader :id
    attr_reader :data
    attr_reader :links

    def initialize(data)
      @data = data
      @links = []
      @id = SecureRandom.uuid
    end

    def link_to(node, directed = false, cost = nil)
      return false if neighbor?(node)
      @links << Link.new(self, node, directed, cost)
      node.link_to(self) unless directed
      true
    end

    def linked_nodes
      @links.map(&:node_end)
    end

    def neighbor?(node)
      @links.any? { |link| link.links_to(node) }
    end

    def visit
      yield self
    end

    def depth_first(&block)
      Traversals.depth_first(self, &block)
    end
  end
end
